/*
 * Copyright © 2008-2011  Peter Colberg and Felix Höfling
 *
 * This file is part of HALMD.
 *
 * HALMD is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see
 * <http://www.gnu.org/licenses/>.
 */

#ifndef HALMD_MDSIM_GPU_POSITIONS_LATTICE_HPP
#define HALMD_MDSIM_GPU_POSITIONS_LATTICE_HPP

#include <lua.hpp>
#include <memory>
#include <vector>

#include <halmd/io/logger.hpp>
#include <halmd/mdsim/box.hpp>
#include <halmd/mdsim/gpu/particle.hpp>
#include <halmd/mdsim/type_traits.hpp>
#include <halmd/utility/profiler.hpp>

namespace halmd {
namespace mdsim {
namespace gpu {
namespace positions {

template <int dimension, typename float_type_>
class lattice
{
public:
    typedef float_type_ float_type;
    typedef gpu::particle<dimension, float_type> particle_type;
    typedef mdsim::box<dimension> box_type;
    typedef typename particle_type::vector_type vector_type;
    typedef typename type_traits<dimension, float>::vector_type gpu_vector_type;
    typedef typename type_traits<dimension, unsigned int>::vector_type index_type;

    static void luaopen(lua_State* L);

    lattice(
        std::shared_ptr<particle_type> particle
      , std::shared_ptr<box_type const> box
      , typename box_type::vector_type const& slab
      , std::shared_ptr<halmd::logger> logger = std::make_shared<halmd::logger>()
    );
    void set();

    typename box_type::vector_type const& slab() const { return slab_; }

private:
    typedef typename particle_type::position_array_type position_array_type;
    typedef typename particle_type::image_array_type image_array_type;

    typedef utility::profiler::accumulator_type accumulator_type;
    typedef utility::profiler::scoped_timer_type scoped_timer_type;

    struct runtime
    {
        accumulator_type set;
    };

    std::shared_ptr<particle_type> particle_;
    std::shared_ptr<box_type const> box_;
    std::shared_ptr<logger> logger_;
    /** slab extents for each direction as fraction of the edge length of the box */
    typename box_type::vector_type slab_;

    /**
     *  assign range of particle positions [first, first+npart) to fcc lattice
     *  of extents 'length' with origin at 'offset'
     */
    template <typename position_iterator>
    void fcc(
        position_iterator first, size_t npart
      , gpu_vector_type const& length, gpu_vector_type const& offset
    );

    /** profiling runtime accumulators */
    runtime runtime_;
};

} // namespace mdsim
} // namespace gpu
} // namespace positions
} // namespace halmd

#endif /* ! HALMD_MDSIM_GPU_POSITIONS_LATTICE_HPP */
